﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2018 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Core;

namespace HeuristicLab.Problems.DataAnalysis {
  public interface ITransformation : IParameterizedNamedItem {
  }

  public interface ITransformation<T> : ITransformation {
    /// <summary>
    /// Checks if the data is valid for the transformation.
    /// Calling <code>Apply</code> or <code>Configure</code> on invalid data will throw an exception.
    /// </summary>
    /// <param name="data"></param>
    /// <param name="errorMessage"></param>
    /// <returns></returns>
    bool Check(IEnumerable<T> data, out string errorMessage);

    /// <summary>
    /// Configure the internal parameter of the transformation for the given data.
    /// </summary>
    /// <param name="data"></param>
    void Configure(IEnumerable<T> data);

    /// <summary>
    /// Transforms the given data based on the transformation's external and internal parameters.
    /// </summary>
    /// <param name="data"></param>
    /// <returns></returns>
    IEnumerable<T> Apply(IEnumerable<T> data);
    /// <summary>
    /// Inverse-transforms the given data based on the transformation's external and internal parameters.
    /// </summary>
    /// <param name="data"></param>
    /// <returns></returns>
    /// <remarks>If the data cannot be inverse transformed, i.e. if the original data was not in the domain of the transformation, 
    /// the returned values can contain invalid values or <code>InverseApply</code> might throw an exception.</remarks>
    IEnumerable<T> InverseApply(IEnumerable<T> data);
  }
}
